@@ -1,5 +1,5 @@
 /* libaudit.h --
- * Copyright 2004-2018,2021 Red Hat Inc.
+ * Copyright 2004-2018,2021-22 Red Hat Inc.
  * All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -23,11 +23,6 @@
 #ifndef _LIBAUDIT_H_
 #define _LIBAUDIT_H_
 
-#ifdef __cplusplus
-extern "C" {
-#endif
-
-
 #include <asm/types.h>
 #include <stdint.h>
 #include <sys/socket.h>
@@ -35,6 +30,17 @@ extern "C" {
 #include <linux/audit.h>
 #include <stdarg.h>
 #include <syslog.h>
+#ifndef __attr_access
+#  define __attr_access(x)
+#endif
+#ifndef __attr_dealloc
+# define __attr_dealloc(dealloc, argno)
+# define __attr_dealloc_free
+#endif
+
+#ifdef __cplusplus
+extern "C" {
+#endif
 
 
 /* Audit message types as of 2.6.29 kernel:
@@ -598,14 +604,16 @@ void set_aumessage_mode(message_t mode, debug_message_t debug);
 typedef enum { GET_REPLY_BLOCKING=0, GET_REPLY_NONBLOCKING } reply_t;
 extern int  audit_open(void);
 extern void audit_close(int fd);
-extern int  audit_get_reply(int fd, struct audit_reply *rep, reply_t block, 
+extern int  audit_get_reply(int fd, struct audit_reply *rep, reply_t block,
 		int peek);
 extern uid_t audit_getloginuid(void);
 extern int  audit_setloginuid(uid_t uid);
 extern uint32_t audit_get_session(void);
 extern int  audit_detect_machine(void);
 extern int audit_determine_machine(const char *arch);
-extern char *audit_format_signal_info(char *buf, int len, char *op, struct audit_reply *rep, char *res);
+extern char *audit_format_signal_info(char *buf, int len, char *op,
+			struct audit_reply *rep, char *res)
+			__attr_access ((__write_only__, 1, 2));
 
 /* Translation functions */
 extern int        audit_name_to_field(const char *field);
@@ -626,9 +634,9 @@ extern const char *audit_operator_to_symbol(int op);
 extern int        audit_name_to_errno(const char *error);
 extern const char *audit_errno_to_name(int error);
 extern int        audit_name_to_ftype(const char *name);
-extern const char *audit_ftype_to_name(int ftype); 
+extern const char *audit_ftype_to_name(int ftype);
 extern int        audit_name_to_fstype(const char *name);
-extern const char *audit_fstype_to_name(int fstype); 
+extern const char *audit_fstype_to_name(int fstype);
 extern void audit_number_to_errmsg(int errnumber, const char *opt);
 
 /* AUDIT_GET */
@@ -648,7 +656,8 @@ extern int  audit_set_backlog_limit(int fd, uint32_t limit);
 int audit_set_backlog_wait_time(int fd, uint32_t bwt);
 int audit_reset_lost(int fd);
 int audit_reset_backlog_wait_time_actual(int fd);
-extern int  audit_set_feature(int fd, unsigned feature, unsigned value, unsigned lock);
+extern int  audit_set_feature(int fd, unsigned feature, unsigned value,
+			      unsigned lock);
 extern int  audit_set_loginuid_immutable(int fd);
 
 /* AUDIT_LIST_RULES */
@@ -675,10 +684,15 @@ extern int audit_delete_rule_data(int fd, struct audit_rule_data *rule,
                                   int flags, int action);
 
 /* The following are for standard formatting of messages */
-extern int audit_value_needs_encoding(const char *str, unsigned int size);
-extern char *audit_encode_value(char *final,const char *buf,unsigned int size);
+extern int audit_value_needs_encoding(const char *str, unsigned int size)
+	__attr_access ((__read_only__, 1, 2));
+extern char *audit_encode_value(char *final,const char *buf,unsigned int size)
+	__attr_access ((__write_only__, 1))
+	__attr_access ((__read_only__, 2, 3));
 extern char *audit_encode_nv_string(const char *name, const char *value,
-	unsigned int vlen);
+	unsigned int vlen)
+	__attr_access ((__read_only__, 2, 3))
+	__attr_dealloc_free;
 extern int audit_log_user_message(int audit_fd, int type, const char *message,
         const char *hostname, const char *addr, const char *tty, int result);
 extern int audit_log_user_comm_message(int audit_fd, int type,
@@ -687,8 +701,8 @@ extern int audit_log_user_comm_message(int audit_fd, int type,
 extern int audit_log_acct_message(int audit_fd, int type, const char *pgname,
         const char *op, const char *name, unsigned int id,
         const char *host, const char *addr, const char *tty, int result);
-extern int audit_log_user_avc_message(int audit_fd, int type, 
-	const char *message, const char *hostname, const char *addr, 
+extern int audit_log_user_avc_message(int audit_fd, int type,
+	const char *message, const char *hostname, const char *addr,
 	const char *tty, uid_t uid);
 extern int audit_log_semanage_message(int audit_fd, int type,
 	const char *pgname, const char *op, const char *name, unsigned int id,
